<?php 

include("eqsetitem.php");

	/**
	 * Holds the information on one complete item set 
	 */
class eqitemset {

	var $FORM_TARGET   = "/wiki/Special:ItemSets";
	var $MAX_ITEMCOUNT = 50;

	var $setid    = null;
	var $setname  = null;
	var $setitems = array();
	var $deleteditems = array();

	var $total_stats    = array();
	var $total_effects  = array();
	var $total_skills   = array();
	var $total_bardmods = array();
	var $total_extradmg = array();
	var $total_banedmg  = array();
	var $total_haste    = null;

		/* Holds the stat differences for comparing sets */
	var $diff_stats    = array();
	var $diff_effects  = array();
	var $diff_skills   = array();
	var $diff_bardmods = array();
	var $diff_extradmg = array();
	var $diff_banedmg  = array();
	var $diff_haste    = null;

	var $isvalid  = false;

		/**
		 * Loads all the item data currently in the set.
		 */
	public function loadItemData () {
		foreach ($this->setitems as $index => $item) {
			$item->item = loadEqItem($item->itemid);
		}
	}

		/**
		 * Compares this set to another set. Computes and stores the differences
		 * for later use.
		 */
	public function compareSet ($itemset) {
		$this->clearDifferences();
		if (!$itemset) return;
 
		$this->diff_stats    = $this->compareSetArray($this->total_stats,    $itemset->total_stats);
		$this->diff_skills   = $this->compareSetArray($this->total_skills,   $itemset->total_skills);
		$this->diff_bardmods = $this->compareSetArray($this->total_bardmods, $itemset->total_bardmods);
		$this->diff_extradmg = $this->compareSetArray($this->total_extradmg, $itemset->total_extradmg);
		$this->diff_banedmg  = $this->compareSetArray($this->total_banedmg,  $itemset->total_banedmg);

		if ($this->total_haste || $itemset->total_haste) {
			$this->diff_haste = $this->total_haste - $itemset->total_haste;
		}
	}

		/**
		 * Helper function for comparing stats from two array.
		 * Returns an array of differences between the given two arrays.
		 */
	private function compareSetArray ($array1, $array2) {
		$result = array();

		foreach ($array1 as $stat => $value) {
			$result[$stat] = $value - $array2[$stat];
		}

		foreach ($array2 as $stat => $value) {
			if (!isset($result[$stat])) $result[$stat] = $array1[$stat] - $value;
		}

		return $result;
	}

		/**
		 * Clears all currently set differences.
		 */
	private function clearDifferences() {
		$this->diff_stats    = array();
		$this->diff_effects  = array();
		$this->diff_skills   = array();
		$this->diff_bardmods = array();
		$this->diff_extradmg = array();
		$this->diff_banedmg  = array();
		$this->diff_haste    = null;
	}

		/**
		 * Attempts to load the item set with the given id from the database.
		 * Returns false on any error.
		 */
	public function loadSet ($id) {

		if (!$id) return false;
		$query = "SELECT * FROM eqitemsets WHERE id=".$id;

		$result = $this->doQuery($query);
		if (!$result) return false;

		$row = mysql_fetch_assoc($result);
		if (!$row) return false;

		$this->setid   = $row['id'];
		$this->setname = $row['name'];
		$this->isvalid = true;
	
		return $this->loadSetItems();
	}

		/**
		 * Attempts to load the item set with the given name from the database.
		 * Returns false on any error.
		 */
	public function loadSetByName ($name) {

		if (!$name) return false;
		$query = "SELECT * FROM eqitemsets WHERE name='". $name ."' LIMIT 1";

		$result = $this->doQuery($query);
		if (!$result) return false;

		$row = mysql_fetch_assoc($result);
		if (!$row) return false;

		$this->setid   = $row['id'];
		$this->setname = $row['name'];
		$this->isvalid = true;
	
		return $this->loadSetItems();
	}

		/**
		 * Attempts to load the set's item from the database.
		 * Returns false on any error.
		 */
	private function loadSetItems () {
		global $itemset_fields;
		$query = "SELECT * FROM eqsetitems WHERE setid=".$this->setid;

		$result = $this->doQuery($query);
		if (!$result) return false;

		while (($row = mysql_fetch_assoc($result))) {
			$newitem = new eqsetitem;
			$newitem->fromAssoc($row);
			$this->setitems[] = $newitem;
		}

		return true;
	}

		/**	
		 * Sets the item set data from the given form request arrays.
		 */
	public function setFromRequest ($setid, $setname, $itemids, $recordids) {
		$this->setid   = $setid;
		$this->setname = $setname;

		foreach ($itemids as $index => $itemid) {
			$id = $recordids[$index];

			if ($itemid) {
				$this->setitems[] = createSetItem($id, $itemid, $this->setid);
			}
			else if ($id) {
				$this->deleteditems[] = createSetItem($id, $itemid, $this->setid);
			}
		}

		$this->isvalid = true;
	}

	public function deleteSet () {
		if ($this->setid == null) return false;
	
		$query = "DELETE FROM eqitemsets WHERE id=" . $this->setid . ";";
		$result = $this->doQuery($query);
		if (!$result) return false;

		return true;
	}

		/**
		 * Attempts to save the set's items to the database.
		 * Returns false on any error.
		 */
	public function saveSet () {

			/* If there is no setid yet we must create a new set */
		if ($this->setid == null) return $this->saveNewSet();

			/* Ensure we have something to save */
		if (!$this->isvalid) return false;

			/* Update the set record */
		$query = "UPDATE eqitemsets SET name='". $this->setname ."' WHERE id=". $this->setid;
		$result = $this->doQuery($query);
		if (!$result) return false;

			/* Save the set items */
		return $this->saveSetItems();
	}

		/**
		 * Saves the current set data to a new record.
		 * Returns false on any error.
		 */
	public function saveNewSet () {

			/* Create the new set record */
		$query = "INSERT INTO eqitemsets SET name='". $this->setname ."'";
		$result = $this->doQuery($query);
		if (!$result) return false;

			/* Obtain the new set id */
		$this->setid = mysql_insert_id();
		if (!$this->setid) return false;

		$this->updateItemSetId();

		$this->isvalid = true;

			/* Save the set items */
		return $this->saveSetItems();
	}

		/**
		 * Updates all child items of the set with the current setid 
		 */
	private function updateItemSetId() {
		foreach ($this->setitems as $index => $setitem) {
			$setitem->setid = $this->setid;
		}

		foreach ($this->deleteditems as $index => $setitem) {
			$setitem->setid = $this->setid;
		}
	}

		/**
		 * Saves the current set items to the database.
		 * Returns false on any error.
		 */
	public function saveSetItems () {

			/* Update or insert item records as needed */
		for ($c = 0; $c < count($this->setitems); $c++) {
			$setitem = $this->setitems[$c];

				/* A null id indicates a new record */
			if ($setitem->id == null) {
				$query = "INSERT INTO eqsetitems SET setid=".$setitem->setid.", itemid=".$setitem->itemid;
				$result = $this->doQuery($query);
				if ($result) $this->setitems[$c]->id = mysql_insert_id();
			}
				/* Update an existing record */
			else {
				$query = "UPDATE eqsetitems SET setid=".$setitem->setid.", itemid=".$setitem->itemid." WHERE id=".$setitem->id;
				$result = $this->doQuery($query);
			}
		}

			/* Delete any records marked for deletion */
		foreach ($this->deleteditems as $index => $setitem) {
			$query = "DELETE FROM eqsetitems WHERE id=".$setitem->id;
			$result = $this->doQuery($query);
		}

		return true;
	}

		/**
		 * Returns a summary string of the set's total statistics. 
		 * Stats are seperated by the given linefeed string.
		 */
	public function getStatSummary ($lf = "\n") {
		$result  = $this->formatTotalStatString("purity", "Purity:", $lf);
		$result .= $this->formatTotalStatString("ac", "AC:", $lf);
		$result .= $this->formatTotalStatString("astr", "STR:", $lf);
		$result .= $this->formatTotalStatString("asta", "STA:", $lf);
		$result .= $this->formatTotalStatString("adex", "DEX:", $lf);
		$result .= $this->formatTotalStatString("aagi", "AGI:", $lf);
		$result .= $this->formatTotalStatString("awis", "WIS:", $lf);
		$result .= $this->formatTotalStatString("aint", "INT:", $lf);
		$result .= $this->formatTotalStatString("acha", "CHA:", $lf);
		$result .= $this->formatTotalStatString("hp", "HP:", $lf);
		$result .= $this->formatTotalStatString("mana", "MANA:", $lf);
		$result .= $this->formatTotalStatString("endur", "END:", $lf);

		return $result;
	}

	public function getHeroicSummary ($lf = "\n") {
		$result = "";

		$result .= $this->formatTotalStatString("heroic_str", "Heroic Str:", $lf);
		$result .= $this->formatTotalStatString("heroic_sta", "Herioc Sta:", $lf);
		$result .= $this->formatTotalStatString("heroic_dex", "Heroic Dex:", $lf);
		$result .= $this->formatTotalStatString("heroic_agi", "Heroic Agi:", $lf);
		$result .= $this->formatTotalStatString("heroic_wis", "Heroic Wis:", $lf);
		$result .= $this->formatTotalStatString("heroic_int", "Heroic Int:", $lf);
		$result .= $this->formatTotalStatString("heroic_cha", "Heroic Cha:", $lf);

		return $result;

	}

		/**
		 * Returns a summary string of the set's total resistances. 
		 * Stats are seperated by the given linefeed string.
		 */
	public function getResistSummary ($lf = "\n") {
		$result  = $this->formatTotalStatString("pr", "SV POISON:", $lf);
		$result .= $this->formatTotalStatString("mr", "SV MAGIC:", $lf);
		$result .= $this->formatTotalStatString("dr", "SV DISEASE:", $lf);
		$result .= $this->formatTotalStatString("fr", "SV FIRE:", $lf);
		$result .= $this->formatTotalStatString("cr", "SV COLD:", $lf);
		$result .= $this->formatTotalStatString("svcorruption", "SV CORRUPTION:", $lf);

		return $result;
	}

		/**
		 * Returns a summary string of the set's total mods/mod2s. 
		 * Stats are seperated by the given linefeed string.
		 */
	public function getModSummary ($lf = "\n") {
		$result  = $this->formatTotalStatString("regen", "Regen:", $lf);
		$result .= $this->formatTotalStatString("manaregen", "Mana Regen:", $lf);
		$result .= $this->formatTotalStatString("enduranceregen", "End Regen:", $lf);
		$result .= $this->formatTotalStatString("accuracy", "Accuracy:", "%".$lf);
		$result .= $this->formatTotalStatString("attack", "Attack:", $lf);
		$result .= $this->formatTotalStatString("avoidance", "Avoidance:", " ");
		$result .= $this->formatTotalStatString("stunresist", "Stun Resist:", " ");
		$result .= $this->formatTotalStatString("combateffects", "Combat Effects:", $lf);
		$result .= $this->formatTotalStatString("strikethrough", "Strikethrough:", "%".$lf);
		$result .= $this->formatTotalStatString("damageshield", "Damage Shield:", $lf);
		$result .= $this->formatTotalStatString("dotshielding", "DoT Shielding:", "%".$lf);
		$result .= $this->formatTotalStatString("shielding", "Shielding:", "%".$lf);
		$result .= $this->formatTotalStatString("spelldmg", "Spell Dmg:", $lf);
		$result .= $this->formatTotalStatString("healamt", "Heal Amt:", $lf);
		$result .= $this->formatTotalStatString("dsmitigation", "DS Mit:", $lf);
		$result .= $this->formatTotalStatString("clairvoyance", "Clairvoyance:", $lf);

		if ($this->total_haste < 0)
			$result .= "Haste:" . $this->total_haste."%" . $lf;
		elseif ($this->total_haste > 0)
			$result .= "Haste:+" . $this->total_haste."%" . $lf;

		return $result;
	}

		/**
		 * Returns a summary string of the set's maximum skill values.
		 * Stats are seperated by the given linefeed string.
		 */
	public function getSkillSummary ($lf = "\n") {
		$result = "";

		foreach ($this->total_skills as $skillnumber => $skillvalue) {
			$result .= getEqSkillText($skillnumber) . ":";
			if ($skillvalue > 0) $result .= "+";
			$result .= $skillvalue . "%" . $lf;
		}

		return $result;
	}

		/**
		 * Returns a summary string of the set's bard mod values.
		 * Stats are seperated by the given linefeed string.
		 */
	public function getBardModSummary ($lf = "\n") {
		$result = "";

		foreach ($this->total_bardmods as $bardtype => $skillvalue) {
			$result .= getEqItemTypeText($bardtype) . ":";
			$result .= $skillvalue. $lf;
		}

		return $result;
	}

		/**
		 * Returns a summary string of the set's extra damage values.
		 * Stats are seperated by the given linefeed string.
		 */
	public function getExtraDmgSummary ($lf = "\n") {
		$result = "";

		foreach ($this->total_extradmg as $type => $value) {
			$result .= getEqSkillText($type) . " DMG:";
			$result .= $value . $lf;
		}

		return $result;
	}

		/**
		 * Formats the given total field and returns the formatted string:
		 *	 0 : Nothing is return
		 *	>0 : "$Header+$Value$Footer"
		 *	<0 : "$Header$Value$Footer"
		 */
	public function formatTotalStatString ($field, $header, $footer="") {

		if ($this->total_stats[$field] > 0)
			return $header . "+" . $this->total_stats[$field] . $footer;
		else if ($this->total_stats[$field] < 0)
			return $header . $this->total_stats[$field] . $footer;

		return "";
	}

		/**
		 * Gathers the total stats for the item set.
		 */
	public function computeTotals () {
		global $sum_itemset_fields, $skills;
		if (!$this->isvalid) return;

		foreach ($this->setitems as $index => $item) {
			if (!$item->item) continue;

				/* Compute the regular total stat fields */
			foreach ($sum_itemset_fields as $field => $title) {
				$value = $item->item->getField($field);
				if ($value) $this->total_stats[$field] += $value;
			}

				/* Check other stats */
			if ($item->item->hasSkillMod())  $this->computeSkillTotal($item->item);
			if ($item->item->isInstrument()) $this->computeBardModTotal($item->item);
			if ($item->item->getField("extradmgamt")) $this->computeExtraDmgTotal($item->item);

			$curhaste = $item->item->getHaste();
			if ($curhaste > $this->total_haste) $this->total_haste = $curhaste;
		}
	}

		/** 
		 * Helper function to compute skill totals for an item 
		 */
	private function computeSkillTotal ($item) {
		$skillnumber  = $item->getField("skillmodtype");
		$itemskill    = $item->getSkillModValue();
		$currentskill = $this->total_skills[$skillnumber];

		if (!$currentskill || $currentskill < $itemskill) {
			$this->total_skills[$skillnumber] = $itemskill;
		}
	}

		/** 
		 * Helper function to compute bard mod totals for an item 
		 */
	private function computeBardModTotal ($item) {
		$type         = $item->getField("bardtype");
		$value        = $item->getField("bardvalue");
		$currentvalue = $this->total_bardmods[$type];

		if (!$currentvalue || $currentvalue < $value) {
			$this->total_bardmods[$type] = $value;
		}
	}

		/** 
		 * Helper function to compute extra damage totals for an item 
		 */
	private function computeExtraDmgTotal ($item) {
		$type  = $item->getField("extradmgskill");
		$value = $item->getField("extradmgamt");

		$this->total_extradmg[$type] += $value;
	}

		/**
		 * Performs a simple query and returns the result.
		 * Used for debugging purposes.
  		 */
	private function doQuery ($query) {
		global $wgOut;

		//$wgOut->addHTML("Query: ".$query."<br/>\n");
		return mysql_query($query);
	}

}

global $sum_itemset_fields;
$sum_itemset_fields = array(

	"accuracy"		=> "Accuracy",
	"aagi"			=> "Agility",
	"ac"			=> "Armor Class",
	"attack"		=> "Attack",
	"avoidance"		=> "Avoidance",
	"acha"			=> "Charisma",
	"clairvoyance"		=> "Clairvoyance",
	"combateffects"		=> "Combat Effects",
	"damageshield"		=> "Damage Shield",
	"adex"			=> "Dexterity",
	"dotshielding"		=> "DoT Shielding",
	"endur"			=> "Endurance",
	"enduranceregen"	=> "Endurance Regen",
	"guildfavor"		=> "Guild Tribute",
	"healamt"		=> "Heal Amt",
	"heroic_agi"		=> "Heroic Agi",
	"heroic_cha"		=> "Heroic Cha",
	"heroic_dex"		=> "Heroic Dex",
	"heroic_int"		=> "Heroic Int",
	"heroic_sta"		=> "Heroic Sta",
	"heroic_str"		=> "Heroic Str",
	"heroic_wis"		=> "Heroic Wis",
	"hp"			=> "Hit Points",
	"aint"			=> "Intelligence",
	"mana"			=> "Mana",
	"manaregen"		=> "Mana Regen",
	"purity"		=> "Purity",
	"regen"			=> "Regen",
	"cr"			=> "Resist Cold",
	"svcorruption"		=> "Resist Corruption",
	"dr"			=> "Resist Disease",
	"fr"			=> "Resist Fire",
	"mr"			=> "Resist Magic",
	"pr"			=> "Resist Poison",
	"shielding"		=> "Shielding",
	"spelldmg"		=> "Spell Dmg",
	"spellshield"		=> "Spell Shield",
	"asta"			=> "Stamina",
	"astr"			=> "Strength",
	"strikethrough"		=> "Strikethrough",
	"stunresist"		=> "Stun Resist",
	"totalstats"		=> "Total Stats",
	"totalresists"		=> "Total Resists",
	"favor"			=> "Tribute",
	"price"			=> "Vendor Price",
	"weight" 		=> "Weight",
	"awis"			=> "Wisdom",
);


?>
